package logic
{
import com.adobe.xml.syndication.generic.Atom10Item;
import com.adobe.xml.syndication.generic.Excerpt;
import com.adobe.xml.syndication.generic.FeedFactory;
import com.adobe.xml.syndication.generic.RSS10Item;
import com.adobe.xml.syndication.generic.RSS20Item;

import data.RSSInfo;

import flash.data.SQLConnection;
import flash.data.SQLResult;
import flash.data.SQLStatement;
import flash.events.ErrorEvent;
import flash.events.Event;
import flash.events.IOErrorEvent;
import flash.events.SQLErrorEvent;
import flash.events.SQLEvent;
import flash.filesystem.File;
import flash.net.URLLoader;
import flash.net.URLRequest;
import flash.net.navigateToURL;

import mx.controls.Alert;
import mx.controls.dataGridClasses.DataGridColumn;
import mx.events.CloseEvent;
import mx.events.FlexEvent;
import mx.events.ListEvent;
import mx.formatters.DateFormatter;
import mx.managers.PopUpManager;

import util.StringUtils;
    
public class RSSReaderLogic extends Logic {
//--------------------------------------
//  Variables
//--------------------------------------
    
    /** 日付フォーマッタ */
    private var dateFormatter:DateFormatter;

//--------------------------------------
//  Constructor
//--------------------------------------
    
    /**
     * コンストラクタ
     */
    public function RSSReaderLogic():void {
        super();
        
        // 日付フォーマッタの構築 
        dateFormatter = new DateFormatter();
        dateFormatter.formatString = "YYYY/MM/DD JJ:NN:SS";
    }

//--------------------------------------
//  Initialization
//--------------------------------------
    /**
     * onCreationCompleteHandler
     */
    override protected function onCreationCompleteHandler(event:FlexEvent):void {
        trace("onCreationCompleteHandler");
        //データベースを開きます。存在しない場合は新規作成されます
        var file:File = File.applicationDirectory.resolvePath("rss.db");
        
        //SQLConnectionを使用して、ローカルデータベースと接続します。
        view.connection = new SQLConnection();
        
        //イベント登録
        view.connection.addEventListener(SQLEvent.OPEN, connectionOpenHandler);
        view.connection.addEventListener(SQLErrorEvent.ERROR, connectionErrorHandler);
        
        //open()によってデータベースを開きます。
        view.connection.open(file); 
    }

//--------------------------------------
//  UI Event Handler
//--------------------------------------

    /**
     * RSSフィードの取得
     * 
     * @param event ListEventオブジェクト
     */
    public function getRSSFeed(event:ListEvent):void {
        trace("getRSSFeed");
        // URLRequestオブジェクトの生成
        var urlRequest:URLRequest = new URLRequest();
        
        // クリックされたURLを取得し、URLRequestオブジェクトにセット
        urlRequest.url = RSSInfo(event.itemRenderer.data).url;
        
        // URLLoaderオブジェクトの生成
        var urlLoader:URLLoader = new URLLoader();
        
        // データ読み込み完了イベントのイベントハンドラの設定
        urlLoader.addEventListener(Event.COMPLETE, loadRssFeedCompleteHandler);
        
        // IOエラーイベントハンドラの設定
        urlLoader.addEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);
        
        // URLLoaderオブジェクトにURLRequestオブジェクトを関連付けて、通信処理開始
        urlLoader.load(urlRequest);
        
        // いったん全画面を利用できなくする
        view.enabled = false;
    }

    /**
     * RSS読み込み完了処理
     * 
     * @param event Eventオブジェクト
     */
    private function loadRssFeedCompleteHandler(event:Event):void {
        trace("loadRssFeedCompleteHandler");
        try {
            // AdobeのライブラリのFeedFactoryを用いて、XMLをRSSフィードとして取得
            view.rss = FeedFactory.getFeedByString(event.target.data);
            
            // RSSからアイテムのリストを取り出す
            view.feedItemList = view.rss.items;
        } catch(e:Error) {
            // 型が判別できない場合処理しない。
            // ライブラリの制限で、RSSが正しく取得できない場合があります
            trace(e.message);
            view.feedItemList = null;
            Alert.show("RSSを正しく取得できませんでした。");
        }
        
        // 全画面を利用できるようにする
        view.enabled = true;
    }
            
    /**
     * RSS読み込み失敗処理
     * 
     * @param event Eventオブジェクト
     */
    private function ioErrorHandler(event:ErrorEvent):void {
        trace("ioErrorHandler");
        trace(event);

        Alert.show("ネットワークエラー。RSSのURLを確認したりして下さい。");

        // 全画面を利用できるようにする
        view.enabled = true;
    }

    /**
     * Feed概要の表示
     * 
     * @param event Eventオブジェクト
     */
    public function showFeed(event:Event):void {
        trace("showFeed");
        // 選択された記事
        var myItem:Object = view.feedList.selectedItem;
        
        // 記事の概要
        var excerptValue:String = "";
        
        // 取得したRSSの型を判定し、概要を取得
        if(myItem is RSS20Item) {
            trace("RSS20Item");
            excerptValue = Excerpt(RSS20Item(myItem).excerpt).value;
        } else if(myItem is RSS10Item) {
            trace("RSS10Item");
            excerptValue = Excerpt(RSS10Item(myItem).excerpt).value;
        } else if(myItem is Atom10Item) {
            trace("Atom10Item");
            excerptValue = Excerpt(Atom10Item(myItem).excerpt).value;
        }
        
        // 概要を画面に表示
        view.myFeed.htmlText = StringUtils.crlf2empty(excerptValue);
    }
    
    /**
     * Feedのブラウザでの表示
     * 
     * @param event Eventオブジェクト
     */
    public function showInBrowser(event:Event):void {
        trace("showInBrowser");
        // 選択されたRSSアイテム
        var myItem:Object = view.feedList.selectedItem;

        // RSSアイテムのlink先
        var link:String = "";
        
        // 取得したRSSの型を判定し、概要を取得
        if(myItem is RSS20Item) {
            trace("RSS20Item");
            link = RSS20Item(myItem).link;
        } else if(myItem is RSS10Item) {
            trace("RSS10Item");
            link = RSS10Item(myItem).link;
        } else if(myItem is Atom10Item) {
            trace("Atom10Item");
            link = Atom10Item(myItem).link;
        }
        
        // URLRequestオブジェクトの作成
        var urlRequest:URLRequest = new URLRequest(link);
        
        // 標準ブラウザに対して指定したURLを開くよう指示
        navigateToURL(urlRequest, "_blank");
    }
    
//--------------------------------------
//  Sub Window Event Handler
//--------------------------------------
    /** 新規RSSウィンドウへの参照 */
    private var addRSSWidndow:AddRSSWidndow;

    /**
     * 新規RSSウィンドウの表示
     */
    public function showAddFeedWindow():void {
        trace("showAddFeedWindow");
        addRSSWidndow = PopUpManager.createPopUp(view, AddRSSWidndow, true) as AddRSSWidndow;
        addRSSWidndow.addEventListener(Event.REMOVED_FROM_STAGE, addRSSWidndowRemoveFromStageHandler);
    }
    
    /**
     * 新規RSSウィンドウの閉じる処理の後処理
     */
    private function addRSSWidndowRemoveFromStageHandler(event:Event):void {
        trace("addRSSWidndowRemoveFromStageHandler");
        addRSSWidndow = null;
    }

    /**
     * RSS削除処理
     */
    public function removeFeed():void {
        trace("removeFeed");
        var myItem:RSSInfo = RSSInfo(view.rssList.selectedItem);
        
        // 選択されたアイテムが存在する場合、削除処理実行
        if(myItem != null) {
            Alert.show("「" + myItem.name + "」のRSS情報を削除します", "RSS情報削除の確認", Alert.YES|Alert.NO, view, removeAlertHandler, null, Alert.YES);
        }
    }
    
    /**
     * 削除確認処理
     */
    private function removeAlertHandler(event:CloseEvent):void {
        // YESが押されたら、削除処理を実行
        if(event.detail == Alert.YES) {
            deleteRSSInfo(RSSInfo(view.rssList.selectedItem));
        }
    }

//--------------------------------------
//  DB Event Handler
//--------------------------------------
    /**
     * データベースとの接続に成功
     */  
    private function connectionOpenHandler(event:SQLEvent):void {
        trace("connectionOpenHandler");
        // CREATE処理
        createTable();
    }

    /**
     * CREATE処理
     */
    private function createTable():void {
        trace("createTable");
        // テーブルを作成します
        view.statement = new SQLStatement();
        view.statement.sqlConnection = view.connection;
        
        // RSS_INFOテーブルの作成
        view.statement.text =
            "CREATE TABLE IF NOT EXISTS RSS_INFO (" +
            "id INTEGER PRIMARY KEY, " +
            "NAME TEXT, " +
            "URL TEXT" +
            ")";
        
        // イベント登録
        view.statement.addEventListener(SQLEvent.RESULT, statementCreateResultHandler);
        view.statement.addEventListener(SQLErrorEvent.ERROR, statementErrorHandler);
        
        // 実行
        view.statement.execute();
    }

    /**
     * CREATE処理結果
     */
    private function statementCreateResultHandler(event:SQLEvent):void {
        trace("statementCreateResultHandler");
        selectRSSInfo();
    }

    /**
     * SELECT処理
     */
    private function selectRSSInfo():void {
        trace("selectRSSInfo");
        view.statement = new SQLStatement();
        view.statement.sqlConnection =  view.connection;
        view.statement.text =
            "SELECT ID, NAME, URL FROM RSS_INFO";
        
        //イベント登録
        view.statement.addEventListener(SQLEvent.RESULT, selectRSSInfoResultHandler);
        view.statement.addEventListener(SQLErrorEvent.ERROR, statementErrorHandler);
        
        //実行
        view.statement.execute();
    }

    /**
     * SELECT処理結果
     */
    private function selectRSSInfoResultHandler(event:SQLEvent):void {
        trace("selectRSSInfoResultHandler");
        var result:SQLResult = view.statement.getResult();
        
        // ステートメントがデータを返さない場合
        if (result.data == null) {
            return;
        }
        
        // 取得データを反映
        view.rssInfoList = RSSInfo.populateArray(result.data);
    }
    
    /**
     * INSERT処理
     */
    public function insertRSSInfo(rssInfo:RSSInfo):void {
        trace("insertRSSInfo");
        view.statement = new SQLStatement();
        view.statement.sqlConnection =  view.connection;
        view.statement.text =
            "INSERT INTO RSS_INFO(name, url) VALUES(" +
            "'" + rssInfo.name + "'," +
            "'" + rssInfo.url + "')"
            ;

        //イベント登録
        view.statement.addEventListener(SQLEvent.RESULT, insertRSSInfoResultHandler);
        view.statement.addEventListener(SQLErrorEvent.ERROR, statementErrorHandler);
        
        //実行
        view.statement.execute();
    }

    /**
     * INSERT処理結果
     */
    private function insertRSSInfoResultHandler(event:SQLEvent):void {
        trace("insertRSSInfoResultHandler");
        // 成功時はそのまま再検索処理
        selectRSSInfo();
    }
        
    /**
     * DELETE処理
     */
    public function deleteRSSInfo(rssInfo:RSSInfo):void {
        trace("deleteRSSInfo");
        view.statement = new SQLStatement();
        view.statement.sqlConnection =  view.connection;
        view.statement.text = "DELETE FROM RSS_INFO WHERE ID = " + rssInfo.id;

        //イベント登録
        view.statement.addEventListener(SQLEvent.RESULT, deleteRSSInfoResultHandler);
        view.statement.addEventListener(SQLErrorEvent.ERROR, statementErrorHandler);
        
        //実行
        view.statement.execute();
    }

    /**
     * DELETE処理結果
     */
    private function deleteRSSInfoResultHandler(event:SQLEvent):void {
        trace("deleteRSSInfoResultHandler");
        // 成功時はそのまま再検索処理
        selectRSSInfo();
    }

    /**
     * データベースとの接続に失敗
     */
    private function connectionErrorHandler(event:SQLErrorEvent):void {
        trace("connectionErrorHandler");
        // エラー内容を表示
        Alert.show("データベースエラー：" + event.error.message);
    }

    /**
     * ステートメントの実行に失敗
     */
    private function statementErrorHandler(event:SQLErrorEvent):void {
        trace("statementErrorHandler");
        // エラー内容を表示
        Alert.show("データベースエラー：" + event.error.message);
    }        

//--------------------------------------
//  Method
//--------------------------------------

    /**
     * 日付表示セルのフォーマット処理
     * 
     * @param data DataGridの選択されたデータ項目
     * @param column DataGridの列オブジェクト
     * @return フォーマット済みのデータ項目
     */
    public function dateLabelFunction(data:Object, column:DataGridColumn):String {
        var returnString:String;
        var date:Date;

        try {
            // []でオブジェクトの指定したフィールドを取得する
            // ここではcolumn.dataField＝dateなので、data.getDate()を呼んでいる
            date = data[column.dataField] as Date;
            returnString = dateFormatter.format(date);
        } catch(e:Error) {
            // getDate()メソッドが例外を出した場合
            // なんとかエラーオブジェクトから日付を取り出してみる
            date = salvageDateFromError(e);
            returnString = dateFormatter.format(date);
        }
        
        return returnString;
    }
    
    /**
     * エラーオブジェクトから日付を力技で取り出す
     * 
     * @param e Error
     * @return 日付文字列
     */
    private function salvageDateFromError(e:Error):Date {
        // [2008-11-11]形式の日付をエラーメッセージから抜き出す
        // W3CDTF形式の日付の扱いもうまくいっていない模様（corelib）
        var message:String = e.message;
        var index:uint = message.indexOf("[") + 1;
        var tempString:String = message.substr(index, 10);
        
        return new Date(Number(tempString.substr(0, 4)),
                        Number(tempString.substr(5, 2)) - 1, 
                        Number(tempString.substr(8, 2))
                        );
    }

//--------------------------------------
//  View-Logic Binding
//--------------------------------------
    /** 画面 */
    public var _view:RSSReader;
    
    /**
     * 画面を取得します
     */     
    public function get view():RSSReader {
        if (_view == null) {
            _view = super.document as RSSReader;
        }
        return _view;
    }
    
    /**
     * 画面をセットします。
     * 
     * @param view セットする画面
     */
    public function set view(view:RSSReader):void {
        _view = view;
    }
}
}